unit dlretrieve;

{$mode objfpc}

interface

uses
  Classes, SysUtils, FileUtil, SynMemo, LResources, Forms, Controls, Graphics,
  Dialogs, StdCtrls, Buttons, LCLType, ExtCtrls;

type

  { TDLRetrieveForm }

  TDLRetrieveForm = class(TForm)
    DLCancelRetrieveButton: TButton;
    DLGHeaderButton: TButton;
    DLRetrieveAllButton: TButton;
    RecentFileEdit: TLabeledEdit;
    OpenDialog1: TOpenDialog;
    OpenDirectoryButton: TBitBtn;
    OpenRecentFileButton: TBitBtn;
    FileDirectoryEdit: TLabeledEdit;
    SynMemo1: TSynMemo;
    SynMemo2: TSynMemo;
    procedure DLCancelRetrieveButtonClick(Sender: TObject);
    procedure DLGHeaderButtonClick(Sender: TObject);
    procedure DLRetrieveAllButtonClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure OpenDirectoryButtonClick(Sender: TObject);
    procedure OpenRecentFileButtonClick(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

var
  DLRetrieveForm: TDLRetrieveForm;

implementation

uses
  Unit1, math,appsettings
  ,header_utils,StrUtils,dateutils
  ,dlheader //contains the DLHeader code
  ,dlerase
  ,vector
;

{ TDLRetrieveForm }

procedure TDLRetrieveForm.DLGHeaderButtonClick(Sender: TObject);
begin
  DLHeaderForm.ShowModal;
end;

procedure TDLRetrieveForm.DLCancelRetrieveButtonClick(Sender: TObject);
begin
  DLCancelRetrieve:=True;
end;

procedure TDLRetrieveForm.DLRetrieveAllButtonClick(Sender: TObject);
var
  result: String;
  subfix: AnsiString;
  pieces: TStringList;
  LogTime : TDateTime;
  Temperature, Voltage, Darkness : Float;
  TriggerModeNumber: Integer;
  DesiredPieces: Integer;
begin
  pieces := TStringList.Create;
  pieces.Delimiter := ',';

  SynMemo2.Clear;
  RecentFileEdit.Text:='';

  //Ensure that continuous logging mode has not been selected because it interferes with data retrieval on firmware less than feature 30.
  result:=sendget('Lmx');
  TriggerModeNumber:=StrToInt(AnsiMidStr(result,4,1));

  //Warn user that erase cannot be done
  if ( (StrToIntDef(SelectedFeature,0)<30) and ((TriggerModeNumber=1) or (TriggerModeNumber=2))) then begin
    Application.MessageBox('Cannot retrieve database while in continuous trigger mode.'+ sLineBreak+
    'Select trigger mode = Off,'+ sLineBreak+
    ' or one of the "Every x on the hour" modes.', 'Busy logging, cannot retrieve!', MB_ICONEXCLAMATION)  ;
    exit;
  end;

   if ((Length(DLHeaderForm.TZRegionBox.Text)>0) and (Length(DLHeaderForm.TZLocationBox.Text)>0)) then
     begin
     DLCancelRetrieve:=False;
       //Get unit time for comparison
       try
          if (SelectedModel=11) then
            WriteDLHeader('DL-V-Log','DL Retrieve All') //vector model
          else
            WriteDLHeader('DL-Log','DL Retrieve All');// Standard DL model

          //Set the desired pieces to be seen when getting a record
          // There are two extra pieces than normal since the date record is split up
          if SelectedModel=11 then
             DesiredPieces:=14    //vector model
          else
             DesiredPieces:=7;    //other DL models

          for DLCurrentRecord:=1 to DLEStoredRecords do begin
              if DLCancelRetrieve then begin
                 StatusMessage('DL Retrieve cancelled.');
                 break;
               end;
               pieces.DelimitedText := sendget(Format('L4%010.10dx',[DLCurrentRecord-1]));
               pieces.StrictDelimiter := False; //Parse spaces also because date is split up

               SynMemo1.Lines.Clear;
               if (pieces.Count=DesiredPieces) then begin
                 SynMemo1.Lines.Append(Format('Retrieved record: %d / %d',[DLCurrentRecord, DLEStoredRecords]));
                 //writeln(Format('Retrieved record: %d / %d',[DLCurrentRecord, DLEStoredRecords]));//debug
                 Application.ProcessMessages;

                 { Pull in unit time for UTC/Local calculations. }
                 try
                    LogTime:=ScanDateTime('yy-mm-ddhh:nn:ss',pieces.Strings[1]+pieces.Strings[3]);
                 except
                    LogTime:=ScanDateTime('yy-mm-dd hh:nn:ss','01-01-01 01:01:01');//Record something anyway even if local date is messed up
                 end;

                 { Pull in values }
                 Temperature:=StrToFloatDef(AnsiLeftStr(pieces.Strings[5],Length(pieces.Strings[5])-1),0,FPointSeparator);
                 Voltage:=(2.048 + (3.3 * StrToFloatDef(pieces.Strings[6],0,FPointSeparator))/256.0);
                 Darkness:=StrToFloatDef(pieces.Strings[4],0,FPointSeparator);

                 { Write record to logfile }
                 Write(DLRecFile,
                    FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"',LogTime) + //Date UTC
                    FormatDateTime('yyyy-mm-dd"T"hh:nn:ss.zzz";"',dlheader.ptz.GMTToLocalTime(LogTime,DLHeaderForm.TZLocationBox.Text,subfix)) + //Date Local (calculated)
                    FormatFloat('##0.0',Temperature,FPointSeparator) + ';' + //Temperature
                    FormatFloat('0.00',Voltage,FPointSeparator) + ';' + //Voltage
                    FormatFloat('#0.00',Darkness,FPointSeparator) //mpsas value
                 );

                 if SelectedModel=11 then begin
                   Write(DLRecFile,Format(';%d;',[StrToIntDef(pieces.Strings[7],0)]));//Acceleration X
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[8],0)]));//Acceleration Y
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[9],0)]));//Acceleration Z
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[10],0)]));//Magnetic X
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[11],0)]));//Magnetic Y
                   Write(DLRecFile,Format('%d;',[StrToIntDef(pieces.Strings[12],0)]));//Magnetic Z
                   Ax:= -1.0 * StrToFloatDef(pieces.Strings[7],0);
                   Ay:=        StrToFloatDef(pieces.Strings[8],0);
                   Az:=        StrToFloatDef(pieces.Strings[9],0);
                   NormalizeAccel(); //Compute acceleration values (In the future, this may be done inside the PIC)
                   Write(DLRecFile,Format('%0.0f;',[radtodeg(arcsin(-1.0*Ax1))]));
                   //
                   Mx:=        StrToFloatDef(pieces.Strings[10],0);
                   My:= -1.0 * StrToFloatDef(pieces.Strings[11],0);
                   Mz:= -1.0 * StrToFloatDef(pieces.Strings[12],0);
                   ComputeAzimuth();
                   Heading:=radtodeg(arctan2(-1*Mz2,Mx2))+180;
                   Write(DLRecFile,Format('%0.0f',[Heading]));
                 end;

                 writeln(DLRecFile,'');//Finish writing line to logfile
               end
               else begin
                  StatusMessage(Format('Failed: actual count = %d , desired count = %d ',[pieces.Count, DesiredPieces]));
                  SynMemo1.Lines.Append(Format('Failed: actual count = %d , desired count = %d ',[pieces.Count, DesiredPieces]));
                  //break;
               end;
          end;
           SynMemo1.Lines.Clear;
           if DLCancelRetrieve then
              SynMemo1.Lines.Append('Partially retrieved records written to:')
           else
               SynMemo1.Lines.Append('Retrieved records written to:');
           RecentFileEdit.Text:=Format('%s',[LogFileName]);
           SynMemo1.Lines.Append(Format('%s',[LogFileName]));
           DLCancelRetrieve:=False;
       except
             StatusMessage('ERROR! IORESULT: ' + IntToStr(IOResult));
       end;
       Flush(DLRecFile);
       CloseFile(DLRecFile);
     end
   else
       ShowMessage('Enter Time zone information into Header first. '+ sLineBreak+
       'Do this by pressing the Header button, then selecting your timezone.');

end;

procedure TDLRetrieveForm.FormShow(Sender: TObject);
begin
     FileDirectoryEdit.Text:=Format('%s%',[appsettings.LogsDirectory + DirectorySeparator]);
end;

procedure TDLRetrieveForm.OpenDirectoryButtonClick(Sender: TObject);
begin
    OpenDialog1.InitialDir := appsettings.LogsDirectory;
  if OpenDialog1.Execute then
  begin{ TODO : file viewer form2, make something new maybe. }
    SynMemo2.Lines.LoadFromFile(OpenDialog1.Filename);
    //Form2.Memo1.Lines.LoadFromFile(OpenDialog1.Filename);
    //Form2.Show;
  end;

end;

procedure TDLRetrieveForm.OpenRecentFileButtonClick(Sender: TObject);
var
  RecentFilname: String;
begin

  RecentFilname:=RecentFileEdit.Text;

  if FileExists(RecentFilname) then
    SynMemo2.Lines.LoadFromFile(RecentFilname)
    else
       begin
        SynMemo2.Clear;
        SynMemo2.Lines.Append('File:');
        SynMemo2.Lines.Append(format('    %s',[RecentFilname]));
        SynMemo2.Lines.Append('does not exist!');
      end;
end;

initialization
  {$I dlretrieve.lrs}

end.

